/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.risk;

import com.yiring.app.domain.workArea.WorkArea;
import com.yiring.common.domain.BasicEntity;
import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.hibernate.annotations.Comment;
import org.locationtech.jts.geom.Point;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/20 17:19
 */
@Getter
@Setter
@ToString
@SuperBuilder(toBuilder = true)
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@Entity
@Table(name = "BS_RISK_WARN", uniqueConstraints = @UniqueConstraint(columnNames = { "locationNum" }))
@Comment("风险预警")
public class RiskWarn extends BasicEntity implements Serializable {

    @Serial
    private static final long serialVersionUID = 1345477064611048638L;

    @Comment("所属工区")
    @ManyToOne
    @JoinColumn(name = "workArea_id")
    private WorkArea workArea;

    @Comment("安装位置")
    private String install;

    @Comment("经纬度")
    @Column(columnDefinition = "geometry(PointZ,4326)")
    private Point point;

    @Comment("监控参数")
    private String monitoring;

    @Comment("位号")
    private String locationNum;

    @Comment("量程起始值")
    private BigDecimal min;

    @Comment("量程结束值")
    private BigDecimal max;

    @Comment("量程单位")
    private String unit;

    @Comment("启用")
    private Boolean enable;

    @Comment("最新监测值")
    private BigDecimal newestMonitor;

    @Comment("最新预警状态:1-正常,2-预警中")
    private Integer warnStatus;

    @Comment("最新阈值状态:1-低低,2-低,3-高,4-高高,5-熄灭")
    private Integer thresholdStatus;

    @Comment("最新风险等级::1-红色风险,2-橙色风险,3-黄色风险,4-蓝色风险")
    private Integer riskLevel;

    @Comment("最新状态更新时间")
    private LocalDateTime newestUpdateTime;

    @ToString.Exclude
    @Comment("阈值区间信息")
    @Builder.Default
    @OneToMany(cascade = { CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "risk_warn_id")
    private List<Threshold> thresholds = new ArrayList<>();
}
