/* (C) 2022 YiRing, Inc. */
package com.yiring.app.param.location.rule;

import com.yiring.app.util.TimeUtil;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * 报警规则,根据不同类型传入不同参数
 * @author tml
 * @version 1.0
 * @date 2022/4/29 14:12
 */
@ApiModel("RuleParam")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class RuleParam implements Serializable {

    @Serial
    private static final long serialVersionUID = 441241201908785930L;

    @ApiModelProperty(value = "选择的播放语音id", example = "1")
    private Long voiceId;

    @ApiModelProperty(value = "选择的播放语音名称", example = "紧急语音")
    private String voiceName;

    @ApiModelProperty(value = "选择的播放设备id", example = "1")
    private Long playEquipmentId;

    @ApiModelProperty(value = "选择的播放设备名称", example = "语音设备1")
    private String playEquipmentName;

    @ApiModelProperty(value = "静止时长或滞留时间(分钟)", example = "10")
    private Integer duration;

    @ApiModelProperty(value = "触发报警的距离(米)", example = "10")
    private Integer distance;

    private List<TimeAndNumberParam> timeAndNumber;

    private List<TimeAndUserParam> timeAndUser;

    /**
     * 设置全时间生效
     */
    public void setTimeAndNumberAll() {
        Integer number = timeAndNumber.get(0).getNumber();
        List<TimeParam> list = TimeUtil.WEEK
            .stream()
            .map(e -> TimeParam.builder().week(e).beginTime("00:00:00").endTime("23:59:59").build())
            .collect(Collectors.toList());
        TimeAndNumberParam timeAndNumberParam = TimeAndNumberParam.builder().number(number).time(list).build();
        List<TimeAndNumberParam> arrayList = new ArrayList<>();
        arrayList.add(timeAndNumberParam);
        timeAndNumber = arrayList;
    }

    public void setTimeAndUserAll() {
        Set<RelevanceUserParam> user = timeAndUser.get(0).getUser();
        List<TimeParam> list = TimeUtil.WEEK
            .stream()
            .map(e -> TimeParam.builder().week(e).beginTime("00:00:00").endTime("23:59:59").build())
            .collect(Collectors.toList());
        TimeAndUserParam timeAndUserParam = TimeAndUserParam.builder().user(user).time(list).build();
        List<TimeAndUserParam> arrayList = new ArrayList<>();
        arrayList.add(timeAndUserParam);
        timeAndUser = arrayList;
    }
}
