/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.fence;

import com.yiring.app.param.location.fence.FenceRealtimeConditionParam;
import com.yiring.app.param.location.fence.LocationFenceAddParam;
import com.yiring.app.param.location.fence.LocationFenceConditionParam;
import com.yiring.app.param.location.fence.LocationFenceModifyParam;
import com.yiring.app.vo.CodeNameVo;
import com.yiring.app.vo.location.fence.FenceRealtimeVo;
import com.yiring.app.vo.location.fence.LocationFenceGeoVo;
import com.yiring.app.vo.location.fence.LocationFenceSearchVo;
import com.yiring.app.vo.location.fence.LocationFenceVo;
import com.yiring.auth.param.IdsParam;
import com.yiring.common.core.Result;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import javax.validation.Valid;

/**
 * @author tml
 * @version 1.0
 * @date 2022/4/28 13:44
 */
public interface LocationFenceService {
    /**
     * 添加围栏信息
     * @param param 围栏信息
     * @return 是否成功
     */
    Result<String> addLocationFence(LocationFenceAddParam param);

    /**
     * 修改围栏信息
     * @param param 围栏信息
     * @return 是否成功
     */
    Result<String> modifyLocationFence(LocationFenceModifyParam param);

    /**
     * 删除围栏信息
     * @param param 围栏id
     * @return 是否成功
     */
    Result<String> removeLocationFence(IdParam param);

    /**
     * 根据条件分页查询围栏信息
     * @param conditionParam 条件
     * @param pageParam 分页参数
     * @return 是否成功
     */
    Result<PageVo<LocationFenceVo>> findList(LocationFenceConditionParam conditionParam, PageParam pageParam);

    /**
     * 查询围栏的位置信息列表
     * @param conditionParam 条件
     * @return 是否成功
     */
    Result<PageVo<LocationFenceGeoVo>> findGeoList(LocationFenceConditionParam conditionParam);

    /**
     * 根据id查询围栏信息
     * @param param id
     * @return 详细信息
     */
    Result<LocationFenceVo> findById(IdParam param);

    /**
     * 根据名称模糊查询围栏信息
     * @param name 围栏名称
     * @return 围栏信息
     */
    Result<PageVo<LocationFenceSearchVo>> findByNameLike(String name);

    /**
     * 获取地图下拉框
     * @return 地图下拉框
     */
    Result<PageVo<CodeNameVo>> findMapList();

    /**
     * 启用或停用一个围栏
     * @param idParam 围栏id
     * @param enable true:启用 false:停用
     * @return 是否成功
     */
    Result<String> enable(IdParam idParam, Boolean enable);

    /**
     * 批量启用或停用围栏
     * @param idsParam 围栏id集
     * @param enable true:启用 false:停用
     * @return 是否成功
     */
    Result<String> batchEnable(@Valid IdsParam idsParam, Boolean enable);

    /**
     * 查询实时围栏信息
     * @param param 条件
     * @return 是否成功
     */
    Result<PageVo<FenceRealtimeVo>> fenRealtimeList(FenceRealtimeConditionParam param);
}
