/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.rehearsal;

import com.yiring.app.param.rehearsal.RehearsalPlanAddParam;
import com.yiring.app.param.rehearsal.RehearsalPlanConditionParam;
import com.yiring.app.param.rehearsal.RehearsalPlanModifyParam;
import com.yiring.app.param.rehearsal.RehearsalPlanPlaybackParam;
import com.yiring.app.vo.rehearsal.RehearsalPlanRealTimeVo;
import com.yiring.app.vo.rehearsal.RehearsalPlanVo;
import com.yiring.common.core.Result;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import javax.servlet.http.HttpServletResponse;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/10 14:18
 */
public interface RehearsalPlanService {
    /**
     * 添加一条演练计划
     * @param param 演练计划信息
     * @return 是否成功
     */
    Result<String> addOne(RehearsalPlanAddParam param);

    /**
     * 修改一条演练计划
     * @param param 演练计划信息
     * @return 是否成功
     */
    Result<String> modifyOne(RehearsalPlanModifyParam param);

    /**
     * 删除一条演练计划
     * @param idParam 演练计划id
     * @return 是否成功
     */
    Result<String> removeOne(IdParam idParam);

    /**
     * 查询一条演练计划信息
     * @param idParam 演练计划id
     * @return 演练计划信息
     */
    Result<RehearsalPlanVo> findOne(IdParam idParam);

    /**
     * 查询演练计划列表
     * @param conditionParam 条件参数
     * @param pageParam 分页参数
     * @return 演练计划列表
     */
    Result<PageVo<RehearsalPlanVo>> findList(RehearsalPlanConditionParam conditionParam, PageParam pageParam);

    /**
     * 导出演练计划
     * @param conditionParam 条件参数
     * @param response response
     */
    void export(RehearsalPlanConditionParam conditionParam, HttpServletResponse response);

    /**
     * 启动一个演练计划
     * @param idParam 演练计划id
     * @return 是否成功
     */
    Result<String> start(IdParam idParam);

    /**
     * 停止一个演练计划
     * @param idParam 演练计划id
     * @return 是否成功
     */
    Result<String> stop(IdParam idParam);

    /**
     * 查看演练(实时)
     * @param idParam 演练计划id
     * @return 实时信息
     */
    Result<RehearsalPlanRealTimeVo> findRealTime(IdParam idParam);

    /**
     * 查看回放
     * @param param 演练计划id和时间
     * @return 回放信息
     */
    Result<PageVo<RehearsalPlanRealTimeVo>> findPlayback(RehearsalPlanPlaybackParam param);
}
