/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.risk.impl;

import cn.hutool.core.convert.Convert;
import com.yiring.app.domain.risk.RiskHandlePush;
import com.yiring.app.domain.risk.RiskHandlePushRepository;
import com.yiring.app.domain.risk.RiskPush;
import com.yiring.app.domain.risk.RiskPushRepository;
import com.yiring.app.param.risk.RiskHandlePushSaveParam;
import com.yiring.app.service.risk.RiskHandlePushService;
import com.yiring.app.vo.risk.RiskHandlePushVo;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 处置推送
 *
 * @author LJ-2204
 * @date 2022/5/24
 */

@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class RiskHandlePushServiceImpl implements RiskHandlePushService {

    @Resource
    RiskHandlePushRepository riskHandlePushRepository;

    @Resource
    RiskPushRepository riskPushRepository;

    @Resource
    UserRepository userRepository;

    // todo 预警区间限制
    @Override
    public Result<String> add(RiskHandlePushSaveParam param) {
        Optional<RiskPush> riskPushOptional = riskPushRepository.findById(param.getRiskPushId());
        if (riskPushOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "风险规则不存在");
        }
        RiskHandlePush riskHandlePush = Convert.convert(RiskHandlePush.class, param);
        riskHandlePush.setRiskPush(riskHandlePush.getRiskPush());

        String[] split = param.getUserIds().split(",");
        Long[] userIds = Convert.toLongArray(split);
        List<User> users = userRepository.findAllById(Arrays.asList(userIds));
        users.forEach(user -> riskHandlePush.getUsers().add(user));

        riskHandlePushRepository.save(riskHandlePush);
        return Result.ok();
    }

    @Override
    public Result<PageVo<RiskHandlePushVo>> page(PageParam pageParam) {
        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<RiskHandlePush> riskHandlePushes = riskHandlePushRepository.findAll(pageable);
        List<RiskHandlePushVo> riskHandlePushVos = riskHandlePushes
            .stream()
            .map(riskHandlePush -> {
                RiskHandlePushVo riskHandlePushVo = Convert.convert(RiskHandlePushVo.class, riskHandlePush);
                Set<User> users = riskHandlePush.getUsers();
                StringBuilder userName = new StringBuilder();
                users.forEach(user -> userName.append(user.getRealName()).append(","));
                if (userName.length() > 0) {
                    String names = userName.substring(0, userName.length() - 1);
                    riskHandlePushVo.setUsers(names);
                }
                return riskHandlePushVo;
            })
            .collect(Collectors.toList());
        PageVo<RiskHandlePushVo> pageVo = PageVo.build(riskHandlePushVos, riskHandlePushes.getTotalElements());
        return Result.ok(pageVo);
    }

    @Override
    public Result<String> delete(IdParam idParam) {
        Optional<RiskHandlePush> riskHandlePushOptional = riskHandlePushRepository.findById(idParam.getId());
        if (riskHandlePushOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "处置推送不存在");
        }

        riskHandlePushRepository.delete(riskHandlePushOptional.get());
        return Result.ok();
    }

    @Override
    public Result<String> modify(RiskHandlePushSaveParam param, IdParam idParam) {
        Optional<RiskHandlePush> riskHandlePushOptional = riskHandlePushRepository.findById(idParam.getId());
        if (riskHandlePushOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "处置推送不存在");
        }

        RiskHandlePush riskHandlePush = riskHandlePushOptional.get();
        riskHandlePush.setTypes(param.getTypes());
        riskHandlePush.setTimeStart(param.getTimeStart());
        riskHandlePush.setTimeEnd(param.getTimeEnd());

        Optional<RiskPush> riskPushOptional = riskPushRepository.findById(param.getRiskPushId());
        if (riskPushOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "风险规则不存在");
        }
        riskHandlePush.setRiskPush(riskPushOptional.get());

        String[] split = param.getUserIds().split(",");
        Long[] userIds = Convert.toLongArray(split);
        List<User> users = userRepository.findAllById(Arrays.asList(userIds));
        riskHandlePush.getUsers().clear();
        users.forEach(user -> riskHandlePush.getUsers().add(user));
        riskHandlePushRepository.save(riskHandlePush);
        return Result.ok();
    }
}
