/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.visitor.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.location.LocationTag;
import com.yiring.app.domain.location.LocationTagRepository;
import com.yiring.app.domain.visitor.Visitor;
import com.yiring.app.domain.visitor.VisitorRepository;
import com.yiring.app.excel.visitor.VisitorExportExcel;
import com.yiring.app.param.user.UserAddParam;
import com.yiring.app.param.visitor.VisitorParam;
import com.yiring.app.param.visitor.VisitorQueryParam;
import com.yiring.app.service.user.UserAppService;
import com.yiring.app.service.visitor.CarService;
import com.yiring.app.service.visitor.VisitorService;
import com.yiring.app.vo.visitor.VisitorDetailsVo;
import com.yiring.app.vo.visitor.VisitorVo;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IndexParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tzl
 * 2022/4/19 14:12
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class VisitorServiceImpl implements VisitorService {

    @Resource
    VisitorRepository visitorRepository;

    @Resource
    CarService carService;

    @Resource
    UserAppService userAppService;

    @Resource
    UserRepository userRepository;

    @Resource
    LocationTagRepository locationTagRepository;

    @Override
    public Result<VisitorDetailsVo> saveVisitor(VisitorParam visitorParam) {
        //设置创建时间和默认状态
        Visitor visitor = Visitor
            .builder()
            .createTime(LocalDateTime.now())
            .updateTime(LocalDateTime.now())
            .cardSendTime(LocalDateTime.now())
            .build();
        BeanUtils.copyProperties(visitorParam, visitor);
        //创建用户
        UserAddParam user = UserAddParam
            .builder()
            .realName(visitorParam.getVisitorName())
            .mobile(visitorParam.getVisitorMobile())
            .type(User.Type.GUEST)
            .deptId(1524237948868890624L)
            .isSpecial(false)
            .gender(true)
            .uuid(RandomUtil.randomNumbers(8))
            .build();
        userAppService.addUser(user);
        User byMobile = userRepository.findByMobile(visitorParam.getVisitorMobile());
        if (byMobile == null) {
            return Result.no(Status.BAD_REQUEST, "标签卡有误");
        }
        //建立标签卡和访客的关系(发卡操作)
        LocationTag byCode = locationTagRepository.findByCode(visitorParam.getLabelCard());
        byCode.setUser(byMobile);
        locationTagRepository.saveAndFlush(byCode);
        //添加一条访客到访记录
        Visitor visitor1 = visitorRepository.saveAndFlush(visitor);
        VisitorDetailsVo visitorDetailsVo = new VisitorDetailsVo();
        BeanUtils.copyProperties(visitor1, visitorDetailsVo);
        return Result.ok(visitorDetailsVo);
    }

    @Override
    public Result<VisitorDetailsVo> selectVisitor(IndexParam indexParam) {
        Visitor visitor = visitorRepository.findByLabelCardAndDeleted(indexParam.getStr(), "N");
        if (visitor == null) {
            return Result.no(Status.NOT_FOUND);
        }
        VisitorDetailsVo vo = new VisitorDetailsVo();
        BeanUtils.copyProperties(visitor, vo);
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<VisitorVo>> pageVisitor(VisitorQueryParam visitorQueryParam) {
        PageVo<VisitorVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Visitor.Fields.createTime));
        if (Objects.nonNull(visitorQueryParam.getPageNo()) && Objects.nonNull(visitorQueryParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(visitorQueryParam.getPageNo() - 1, visitorQueryParam.getPageSize());
            Page<Visitor> all = visitorRepository.findAll(condition(visitorQueryParam), pageable);
            List<VisitorVo> data = all
                .get()
                .map(visitor -> {
                    VisitorVo vo = new VisitorVo();
                    BeanUtils.copyProperties(visitor, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Visitor> all = visitorRepository.findAll(condition(visitorQueryParam), sort);
            List<VisitorVo> data = all
                .stream()
                .map(visitor -> {
                    VisitorVo vo = new VisitorVo();
                    BeanUtils.copyProperties(visitor, vo);
                    //                    VisitorVo.UserVo userVo = VisitorVo.UserVo
                    //                        .builder()
                    //                        .realName(visitor.getUser().getRealName())
                    //                        .build();
                    //                    vo.setUserVo(userVo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public void exportVisitor(VisitorQueryParam visitorQueryParam, HttpServletResponse response) {
        List<Visitor> visitors = visitorRepository.findAll(condition(visitorQueryParam));
        List<VisitorExportExcel> visitorExportExcels = visitors
            .stream()
            .map(visitor -> {
                VisitorExportExcel visitorExportExcel = new VisitorExportExcel();
                BeanUtils.copyProperties(visitor, visitorExportExcel);
                return visitorExportExcel;
            })
            .toList();

        try (
            DefaultStreamExcelBuilder<VisitorExportExcel> defaultStreamExcelBuilder = DefaultStreamExcelBuilder
                .of(VisitorExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .widths(8)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            defaultStreamExcelBuilder.append(visitorExportExcels);

            String fileName = URLEncoder.encode("访客信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = defaultStreamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出访客信息失败: " + e.getMessage());
        }
    }

    //    @Override
    //    public Result<String> cardSend(String labelCard, IdParam idParam) {
    //        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
    //        if (optional.isEmpty()) {
    //            return Result.no(Status.NOT_FOUND);
    //        }
    //        Visitor visitor = optional.get();
    //        //状态变更为使用中
    //        visitor.setLabelCardStatus("2");
    //        visitor.setCardSendTime(LocalDateTime.now());
    //        visitor.setLabelCard(labelCard);
    //        visitorRepository.saveAndFlush(visitor);
    //        return Result.ok();
    //    }

    //    @Override
    //    public Result<String> cardRec(IdParam idParam) {
    //        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
    //        if (optional.isEmpty()) {
    //            return Result.no(Status.NOT_FOUND);
    //        }
    //        Visitor visitor = optional.get();
    //        visitor.setLabelCardStatus("3");
    //        visitor.setUpdateTime(LocalDateTime.now());
    //        visitor.setCardRecTime(LocalDateTime.now());
    //        visitorRepository.saveAndFlush(visitor);
    //        return Result.ok();
    //    }

    //    @Override
    //    public Result<PageVo<VisitorAppVo>> selectInfo(VisitorAppQueryParam visitorAppQueryParam) {
    //        PageVo<VisitorAppVo> resultVo;
    //        //排序
    //
    //        //分页
    //        Pageable pageable = PageRequest.of(visitorAppQueryParam.getPageNo() - 1, visitorAppQueryParam.getPageSize());
    //        Page<Visitor> all = visitorRepository.findAllByVisitorMobileAndDeletedEquals(
    //            visitorAppQueryParam.getVisitorMobile(),
    //            "N",
    //            pageable
    //        );
    //        List<VisitorAppVo> data = all
    //            .get()
    //            .map(visitor -> {
    //                VisitorAppVo vo = new VisitorAppVo();
    //                vo.setId(visitor.getId());
    //                vo.setCardSendTime(visitor.getCardSendTime());
    //                VisitorAppVo.UserVo userVo = VisitorAppVo.UserVo
    //                    .builder()
    //                    .realName(visitor.getUser().getRealName())
    //                    .mobile(visitor.getVisitorMobile())
    //                    .build();
    //                vo.setUserVo(userVo);
    //                return vo;
    //            })
    //            .collect(Collectors.toList());
    //        resultVo = PageVo.build(data, all.getTotalElements());
    //        return Result.ok(resultVo);
    //    }

    public Specification<Visitor> condition(VisitorQueryParam visitorQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            Join<Visitor, User> join = root.join("user", JoinType.LEFT);
            //查询条件
            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorName())) {
                //访客姓名
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorName),
                        "%" + visitorQueryParam.getVisitorName() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorMobile())) {
                //访客电话
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorMobile),
                        "%" + visitorQueryParam.getVisitorMobile() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorCarNum())) {
                //访客身份证
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorCarNum),
                        "%" + visitorQueryParam.getVisitorCarNum() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getTaskType())) {
                //是否特殊作业
                list.add(
                    criteriaBuilder.like(root.get(Visitor.Fields.taskType), "%" + visitorQueryParam.getTaskType() + "%")
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getIntervieweeId())) {
                //被访人
                list.add(criteriaBuilder.equal(join.get(BasicEntity.Fields.id), visitorQueryParam.getIntervieweeId()));
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getLabelCard())) {
                //标签卡号查询
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.labelCard),
                        "%" + visitorQueryParam.getLabelCard() + "%"
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getCardSendTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getCardSendTimeEnd())
            ) {
                //实际到访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.cardSendTime),
                        visitorQueryParam.getCardSendTimeStart(),
                        visitorQueryParam.getCardSendTimeEnd()
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getCardRecTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getCardRecTimeEnd())
            ) {
                //收卡时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.cardRecTime),
                        visitorQueryParam.getCardRecTimeStart(),
                        visitorQueryParam.getCardRecTimeEnd()
                    )
                );
            }
            Order order = criteriaBuilder.desc(root.get(Visitor.Fields.updateTime));
            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            query.orderBy(order);
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
