/* (C) 2022 YiRing, Inc. */
package com.yiring.app.vo.location.fence;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.yiring.app.domain.location.LocationFenceAlarm;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/20 9:48
 */
@ApiModel("实时围栏报警信息")
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class FenceAlarmRealtimeVo implements Serializable {

    @Serial
    private static final long serialVersionUID = -16864683628344917L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "报警信息id", example = "2")
    private Long id;

    @ApiModelProperty(value = "所在围栏名称", example = "一号围栏")
    private String fenceName;

    @ApiModelProperty(value = "触警人", example = "张三")
    private String user;

    @ApiModelProperty(value = "报警时间", example = "2022-05-20 10:00:00")
    private LocalDateTime startTime;

    @ApiModelProperty(value = "报警类型", example = "超员报警")
    private String alarmType;

    public static FenceAlarmRealtimeVo transform(LocationFenceAlarm locationFenceAlarm) {
        return FenceAlarmRealtimeVo
            .builder()
            .id(locationFenceAlarm.getId())
            .fenceName(locationFenceAlarm.getFence().getName())
            .user(locationFenceAlarm.getUser().getRealName())
            .startTime(locationFenceAlarm.getStartTime())
            .alarmType(locationFenceAlarm.getType().getName())
            .build();
    }
}
