/* (C) 2022 YiRing, Inc. */
package com.yiring.app.vo.location.fence;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.yiring.app.constant.alarm.FenceTypeEnum;
import com.yiring.app.constant.alarm.InformMannerEnum;
import com.yiring.app.domain.location.LocationFenceAlarm;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.stream.Collectors;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/17 15:15
 */
@ApiModel("围栏报警数据管理")
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class FenceAlarmVo implements Serializable {

    @Serial
    private static final long serialVersionUID = 8259241388552022873L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "id", example = "1")
    private Long id;

    @ApiModelProperty(value = "地图id", example = "1")
    private Long mapId;

    @ApiModelProperty(value = "地图名称", example = "总图")
    private String mapName;

    @ApiModelProperty(value = "围栏名称", example = "一号围栏")
    private String fenceName;

    @ApiModelProperty(value = "围栏类别code", example = "1")
    private Integer fenceType;

    @ApiModelProperty(value = "围栏类别名称", example = "普通围栏")
    private String fenceTypeName;

    @ApiModelProperty(value = "报警人员名称", example = "张三")
    private String realName;

    @ApiModelProperty(value = "标签号", example = "BTT1")
    private String tagId;

    @ApiModelProperty(value = "报警开始时间", example = "2022-05-04 13:00:00")
    private LocalDateTime startTime;

    @ApiModelProperty(value = "报警结束时间", example = "2022-05-04 13:00:00")
    private LocalDateTime endTime;

    @ApiModelProperty(value = "报警类型", example = "超员报警")
    private String alarmTypeName;

    @ApiModelProperty(value = "接收人", example = "李四")
    private String recipient;

    @ApiModelProperty(value = "报警状态", example = "报警中")
    private String statusName;

    @ApiModelProperty(value = "通知方式", example = "app消息, 短信, 电话")
    private String informManner;

    public static FenceAlarmVo transform(LocationFenceAlarm fenceAlarm) {
        String recipient = fenceAlarm
            .getPushLogs()
            .stream()
            .map(e -> e.getUser().getRealName())
            .distinct()
            .collect(Collectors.joining(", "));
        String informManner = fenceAlarm
            .getPushLogs()
            .stream()
            .map(e -> InformMannerEnum.getByCode(e.getInformManner()))
            .distinct()
            .collect(Collectors.joining(", "));
        return FenceAlarmVo
            .builder()
            .id(fenceAlarm.getId())
            .mapId(fenceAlarm.getAreaId())
            .fenceName(fenceAlarm.getFence().getName())
            .fenceType(fenceAlarm.getFence().getFenceType())
            .fenceTypeName(FenceTypeEnum.getByCode(fenceAlarm.getFence().getFenceType()))
            .realName(fenceAlarm.getUser().getRealName())
            .tagId(fenceAlarm.getTag().getCode())
            .startTime(fenceAlarm.getStartTime())
            .endTime(fenceAlarm.getEndTime())
            .alarmTypeName(fenceAlarm.getType().getName())
            .recipient(recipient)
            .statusName(fenceAlarm.getStatus().text())
            .informManner(informManner)
            .build();
    }
}
