/* (C) 2022 YiRing, Inc. */
package com.yiring.app.web.location.fence;

import com.yiring.app.constant.alarm.FenceTypeEnum;
import com.yiring.app.domain.alarm.AlarmType;
import com.yiring.app.param.location.fence.FenceRealtimeConditionParam;
import com.yiring.app.param.location.fence.LocationFenceAddParam;
import com.yiring.app.param.location.fence.LocationFenceConditionParam;
import com.yiring.app.param.location.fence.LocationFenceModifyParam;
import com.yiring.app.service.location.fence.LocationFenceService;
import com.yiring.app.vo.CodeNameVo;
import com.yiring.app.vo.location.fence.FenceRealtimeVo;
import com.yiring.app.vo.location.fence.LocationFenceGeoVo;
import com.yiring.app.vo.location.fence.LocationFenceSearchVo;
import com.yiring.app.vo.location.fence.LocationFenceVo;
import com.yiring.auth.param.IdsParam;
import com.yiring.common.core.Result;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import java.util.Objects;
import javax.annotation.Resource;
import javax.validation.Valid;
import org.springframework.data.domain.Sort;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author tml
 * @version 1.0
 * @date 2022/4/28 11:50
 */

@Validated
@SuppressWarnings({ "deprecation" })
@Api(tags = "围栏信息", description = "LocationFence")
@RestController
@RequestMapping("/location/fence")
public class LocationFenceController {

    @Resource
    private LocationFenceService locationFenceService;

    @ApiOperation("添加围栏信息")
    @PostMapping("/addLocationFence")
    public Result<String> addLocationFence(@Valid @RequestBody LocationFenceAddParam param) {
        return locationFenceService.addLocationFence(param);
    }

    @ApiOperation("修改围栏信息")
    @PostMapping("/modifyLocationFence")
    public Result<String> modifyLocationFence(@Valid @RequestBody LocationFenceModifyParam param) {
        return locationFenceService.modifyLocationFence(param);
    }

    @ApiOperation("删除围栏信息")
    @PostMapping("/removeLocationFence")
    public Result<String> removeLocationFence(@Valid IdParam param) {
        return locationFenceService.removeLocationFence(param);
    }

    @ApiOperation("根据条件分页查询围栏信息")
    @PostMapping("/findList")
    public Result<PageVo<LocationFenceVo>> findList(
        @Valid LocationFenceConditionParam conditionParam,
        @Valid PageParam pageParam
    ) {
        if (Objects.isNull(pageParam.getSortField())) {
            pageParam.setSortField(AlarmType.Fields.createTime);
            pageParam.setSortOrder(Sort.Direction.DESC);
        }
        return locationFenceService.findList(conditionParam, pageParam);
    }

    @ApiOperation("根据条件查询围栏地图位置信息")
    @GetMapping("/findGeoList")
    public Result<PageVo<LocationFenceGeoVo>> findGeoList(@Valid LocationFenceConditionParam conditionParam) {
        return locationFenceService.findGeoList(conditionParam);
    }

    @ApiOperation("根据id查询围栏信息")
    @PostMapping("/findById")
    public Result<LocationFenceVo> findById(@Valid IdParam param) {
        return locationFenceService.findById(param);
    }

    @ApiOperation("根据围栏名称模糊查询围栏信息")
    @ApiImplicitParam(value = "围栏名称", example = "生产", required = true, name = "name")
    @PostMapping("/findByNameLike")
    public Result<PageVo<LocationFenceSearchVo>> findByNameLike(String name) {
        if (name == null) {
            name = "";
        }
        return locationFenceService.findByNameLike(name);
    }

    @ApiOperation("查询围栏类型下拉框")
    @GetMapping("/findFenceType")
    public Result<PageVo<CodeNameVo>> findFenceType() {
        List<CodeNameVo> list = FenceTypeEnum.getAll();
        PageVo<CodeNameVo> pageVo = PageVo.build(list, list.size());
        return Result.ok(pageVo);
    }

    @ApiOperation("获取地图下拉框")
    @PostMapping("/findMapList")
    public Result<PageVo<CodeNameVo>> findMapList() {
        return locationFenceService.findMapList();
    }

    @ApiOperation("启用一个围栏")
    @PostMapping("/enable")
    public Result<String> enable(@Valid IdParam idParam) {
        return locationFenceService.enable(idParam, true);
    }

    @ApiOperation("停用一个围栏")
    @PostMapping("/unEnable")
    public Result<String> unEnable(@Valid IdParam idParam) {
        return locationFenceService.enable(idParam, false);
    }

    @ApiOperation("批量启用一个围栏")
    @PostMapping("/batchEnable")
    public Result<String> batchEnable(@Valid IdsParam idsParam) {
        return locationFenceService.batchEnable(idsParam, true);
    }

    @ApiOperation("批量停用一个围栏")
    @PostMapping("/batchUnEnable")
    public Result<String> batchUnEnable(@Valid IdsParam idsParam) {
        return locationFenceService.batchEnable(idsParam, false);
    }

    @ApiOperation("实时查询围栏信息")
    @GetMapping("/fenRealtimeList")
    public Result<PageVo<FenceRealtimeVo>> fenRealtimeList(@Valid FenceRealtimeConditionParam param) {
        return locationFenceService.fenRealtimeList(param);
    }
}
