/* (C) 2022 YiRing, Inc. */
package com.yiring.app.web.location.turnover;

import com.yiring.app.param.perstatistics.PersonnelStatisticsQueryParam;
import com.yiring.app.service.location.turnover.LocationTurnoverService;
import com.yiring.app.vo.perstatistics.PersonnelStatisticsDetailVo;
import com.yiring.app.vo.perstatistics.PersonnelStatisticsEchartsVo;
import com.yiring.app.vo.perstatistics.PersonnelStatisticsVo;
import com.yiring.common.core.Result;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author tzl
 * @version 1.0
 * @description:
 * @date 2022/5/15 9:49
 */
@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@Api(tags = "人员统计", description = "LocationTurnover")
@RestController
@RequestMapping("/locationTurnover/")
public class LocationTurnoverController {

    @Resource
    LocationTurnoverService locationTurnoverService;

    @ApiOperation(value = "分页查询人员统计")
    @GetMapping("pageInfo")
    public Result<PageVo<PersonnelStatisticsVo>> pageInfo(
        @Valid PersonnelStatisticsQueryParam personnelStatisticsQueryParam,
        @Valid PageParam pageParam
    ) {
        return locationTurnoverService.pageInfo(personnelStatisticsQueryParam, pageParam);
    }

    @ApiOperation(value = "查询人员统计图表")
    @GetMapping("listInfo")
    public Result<PageVo<PersonnelStatisticsEchartsVo>> listInfo() {
        return locationTurnoverService.listInfo();
    }

    @ApiOperation(value = "下拉框")
    @GetMapping("indexInfo")
    public Result<PageVo<PersonnelStatisticsVo>> indexInfo(
        @Valid PersonnelStatisticsQueryParam personnelStatisticsQueryParam
    ) {
        return locationTurnoverService.indexInfo(personnelStatisticsQueryParam);
    }

    @ApiOperation(value = "导出", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @PostMapping("export")
    public void exportVideo(
        HttpServletResponse response,
        @Valid PersonnelStatisticsQueryParam personnelStatisticsQueryParam
    ) {
        locationTurnoverService.export(personnelStatisticsQueryParam, response);
    }

    @ApiOperation(value = "查询人员统计详情")
    @GetMapping("selectInfo")
    public Result<PageVo<PersonnelStatisticsDetailVo>> selectInfo(
        @Valid PersonnelStatisticsQueryParam personnelStatisticsQueryParam,
        @Valid PageParam pageParam
    ) {
        return locationTurnoverService.selectInfo(personnelStatisticsQueryParam, pageParam);
    }
}
